/*-----------------------------------------------------------------------------
 * Name:    ADC_STM32303C-EVAL.c
 * Purpose: ADC interface for STM32303C-EVAL board
 * Rev.:    1.01
 *----------------------------------------------------------------------------*/

/* Copyright (c) 2013 - 2015 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

#include "stm32f3xx.h"                  // Device header
#include "Board_ADC.h"                  // ::Board Support:A/D Converter

#define ADC_BITS      12                /* Number of A/D converter bits       */

volatile uint16_t AD_last;              /* Last converted value               */
volatile uint8_t  AD_done;              /* AD conversion done flag            */


/**
  \fn          int32_t ADC_Initialize (void)
  \brief       Initialize buttons
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_Initialize (void) {

  RCC->AHBENR  |= ( 1UL <<   19);         /* Enable GPIOC clock               */
  GPIOC->MODER |= ( 3UL << 2* 1);         /* PC.1  is Analog mode             */

#ifndef __ADC_IRQ
  /* DMA1 Channel1 configuration ---------------------------------------------*/
  RCC->AHBENR  |= (1UL <<  0);            /* enable DMA1 clock                */

  DMA1_Channel1->CMAR  = (uint32_t)&AD_last;    /* Set memory     address     */
  DMA1_Channel1->CPAR  = (uint32_t)&(ADC1->DR); /* Set peripheral address     */
  DMA1_Channel1->CNDTR = 1;               /* Transmit 1 data item             */
  DMA1_Channel1->CCR   = 0x00002522;      /* configure DMA channel            */
  NVIC_EnableIRQ(DMA1_Channel1_IRQn);     /* enable DMA1 Channel Interrupt    */
  DMA1_Channel1->CCR  |= (1UL << 0);      /* enable DMA channel 1             */
#endif

  /* Setup and initialize ADC converter                                       */
  RCC->CR      |= ( 1UL <<  0);           /* Enable Internal high-speed clock */
  RCC->CFGR2   &= (0x1F <<  4);           /* clear ADC12PRES                  */
  RCC->CFGR2   |= (0x12 <<  4);           /* ADC12PRES = PLL clock / 4        */
  RCC->AHBENR  |= ( 1UL << 28);           /* enable periperal clock for ADC1/2*/

  ADC1->CR      = (1UL <<  28);           /* enable ADC VREG                  */
  __NOP();__NOP();__NOP();__NOP();__NOP();__NOP();

  ADC1->CR     |= ( 1UL << 31);           /* start Calibration */
  while (ADC1->CR & ( 1UL << 31)) __NOP();
  ADC1->CR     |= (1UL << 30);            /* enable differential mode */
  ADC1->CR     |= ( 1UL << 31);           /* start Calibration */
  while (ADC1->CR & ( 1UL << 31)) __NOP();

  ADC1->SQR1    = ( 7UL <<  6);           /* 1. conversion = channel 7 */
  ADC1->SQR2    =  0;                     /* Clear register                   */
  ADC1->SQR3    =  0;                     /* Clear register                   */
  ADC1->SQR4    =  0;                     /* Clear register                   */
  ADC1->SMPR1   = ( 4UL << 21);           /* Chn 7 sample time is 19.5 cyc    */
  ADC1->SMPR2   =  0;                     /* Clear register                   */

#ifndef __ADC_IRQ
  ADC1->CFGR   |= ( 3UL <<  0);           /* DMA mode enable, cont. DMA req.  */
#else
  ADC1->IER    |= ( 1UL <<  2);           /* enable for EOC Interrupt         */
  NVIC_EnableIRQ(ADC1_2_IRQn);            /* enable ADC Interrupt             */
#endif

  ADC1->CR     |= ( 1UL <<  0);           /* ADC enable                       */

  return(0);
}


/**
  \fn          int32_t ADC_Uninitialize (void)
  \brief       De-initialize Analog-to-Digital Converter
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_Uninitialize (void) {

	return(0);
}


/**
  \fn          int32_t ADC_StartConversion (void)
  \brief       Start conversion
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_StartConversion (void) {

  AD_done = 0;
  ADC1->CR     |=  (1UL <<  2);

  return(0);
}


/**
  \fn          int32_t ADC_ConversionDone (void)
  \brief       Check if conversion finished
  \returns
   - \b  0: conversion finished
   - \b -1: conversion in progress
*/
int32_t ADC_ConversionDone (void) {

  return (AD_done) ? 0 : -1;
}


/**
  \fn          int32_t ADC_GetValue (void)
  \brief       Get converted value
  \returns
   - <b> >=0</b>: converted value
   - \b -1: conversion in progress or failed
*/
int32_t ADC_GetValue (void) {

  while (!(AD_done));                     /* Wait for Conversion end          */
  AD_done = 0;

  return(AD_last);
}


/**
  \fn          uint32_t ADC_GetResolution (void)
  \brief       Get resolution of Analog-to-Digital Converter
  \returns     Resolution (in bits)
*/
uint32_t ADC_GetResolution (void) {

  return (ADC_BITS);
}

#ifndef __ADC_IRQ
/*----------------------------------------------------------------------------
  DMA IRQ: Executed when a transfer is completet
 *----------------------------------------------------------------------------*/
void DMA1_Channel1_IRQHandler(void) {

  if (DMA1->ISR & (1 << 1)) {             /* TCIF interrupt?                  */
    AD_done = 1;

    DMA1->IFCR  = (1 << 1);               /* clear TCIF interrupt             */
  }
}
#endif


#ifdef __ADC_IRQ
/*----------------------------------------------------------------------------
  A/D IRQ: Executed when A/D Conversion is done
 *----------------------------------------------------------------------------*/
void ADC1_2_IRQHandler(void) {

  if (ADC1->ISR & (1 << 2)) {              /* ADC1 EOC interrupt?              */
    AD_last = ADC1->DR;
    AD_done = 1;

    ADC1->ISR |=  (1 << 2);                /* clear EOC interrupt              */
  }

}
#endif
