/*-----------------------------------------------------------------------------
 * Name:    SADC_STM32373C-EVAL.c
 * Purpose: SADC interface for STM32373C-EVAL board
 * Rev.:    1.01
 *----------------------------------------------------------------------------*/

/* Copyright (c) 2013 - 2015 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

#include "stm32f3xx.h"                  // Device header
#include "Board_ADC.h"                  // ::Board Support:A/D Converter

#define ADC_BITS      16                /* Number of A/D converter bits       */

volatile uint16_t AD_last;              /* Last converted value               */
volatile uint8_t  AD_done;              /* AD conversion done flag            */


/**
  \fn          int32_t ADC_Initialize (void)
  \brief       Initialize buttons
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_Initialize (void) {

  RCC->AHBENR  |= ( 1UL <<   18);         /* Enable GPIOB clock               */
  GPIOB->MODER |= ( 3UL << 2* 1);         /* PB.1  is Analog mode             */

#ifndef __SDADC_IRQ
  /* DMA1 Channel1 configuration ---------------------------------------------*/
  RCC->AHBENR  |= (1UL <<  1);            /* enable DMA2 clock                */

  DMA2_Channel3->CMAR  = (uint32_t)&AD_last;          /* Set memory     addr  */
  DMA2_Channel3->CPAR  = (uint32_t)&(SDADC1->JDATAR); /* Set peripheral addr  */
  DMA2_Channel3->CNDTR = 1;               /* Transmit 1 data item             */
  DMA2_Channel3->CCR   = ((1UL <<  1) |   /* enable Transfer complete IRQ     */
                          (0UL <<  4) |   /* destination read from peripheral */
                          (1UL <<  5) |   /* enable Circular mode             */
                          (1UL <<  8) |   /* set PSIZE to 16Bit               */
                          (1UL << 10) |   /* set MSIZE to 16Bit               */
                          (1UL << 12)  ); /* channel priority medium          */
  NVIC_EnableIRQ(DMA2_Channel3_IRQn);     /* enable DMA2 Channel Interrupt    */
  DMA2_Channel3->CCR  |=  (1UL << 0);     /* enable DMA channel 3             */
#endif

  /* Setup and initialize ADC converter                                       */
  RCC->CFGR &= ~(0x1FUL << 27);
  RCC->CFGR |=  (0x15UL << 27);           /* SDADC1 clock is 6MHz @ 72 MZ     */

  RCC->APB1ENR |= (1UL << 28);            /* enable Power interface clock     */
  PWR->CR      |= (1UL <<  9);            /* enable SD1                       */
  RCC->APB2ENR |= (1UL << 24);            /* enable SDADC1 clock              */

  SDADC1->CR1 &= ~(3UL <<  8);            /* select external reference        */

  SDADC1->CR1 |=  (1UL << 31);            /* Initialization mode request      */
  while ((SDADC1->CR1 & (1UL << 31)) == 0) __NOP(); /* wait for init. mode    */

  SDADC1->CONF0R = (3UL << 26);           /* single ended 0-volt reference    */
  SDADC1->CONFCHR1 &= ~(3UL << (4*5));    /* chn5 uses SDADC_CONFR0           */

  SDADC1->JCHGR  = (1UL << 5);            /* chn5 part of injected group      */

  SDADC1->CR1 &= ~(1UL << 31);            /* exit initialization mode         */

#ifndef __SDADC_IRQ
  SDADC1->CR1  |= (1UL << 16);            /* DMA mode enable                  */
#else
  SDADC1->CR1  |= ( 1UL <<  1);           /* enable injected EOC Interrupt    */
  NVIC_EnableIRQ(SDADC1_IRQn);            /* enable SDADC Interrupt           */
#endif

  SDADC1->CR2 |= (1UL <<  0);             /* enable SDADC                     */
  SDADC1->CR2 |= (1UL <<  4);             /* start calibration                */
  while ((SDADC1->ISR & (1UL <<  0)) == 0) __NOP(); /* wait for CAL finished  */

  return(0);
}


/**
  \fn          int32_t ADC_Uninitialize (void)
  \brief       De-initialize Analog-to-Digital Converter
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_Uninitialize (void) {

  return(0);
}


/**
  \fn          int32_t ADC_StartConversion (void)
  \brief       Start conversion
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_StartConversion (void) {

  AD_done = 0;
  SDADC1->CR2 |=  (1UL << 15);

  return(0);
}


/**
  \fn          int32_t ADC_ConversionDone (void)
  \brief       Check if conversion finished
  \returns
   - \b  0: conversion finished
   - \b -1: conversion in progress
*/
int32_t ADC_ConversionDone (void) {

  return (AD_done) ? 0 : -1;
}


/**
  \fn          int32_t ADC_GetValue (void)
  \brief       Get converted value
  \returns
   - <b> >=0</b>: converted value
   - \b -1: conversion in progress or failed
*/
int32_t ADC_GetValue (void) {

  while (!(AD_done));                     /* Wait for Conversion end          */
  AD_done = 0;

  return(AD_last);
}


/**
  \fn          uint32_t ADC_GetResolution (void)
  \brief       Get resolution of Analog-to-Digital Converter
  \returns     Resolution (in bits)
*/
uint32_t ADC_GetResolution (void) {

  return (ADC_BITS);
}

#ifndef __SDADC_IRQ
/*  DMA IRQ: Executed when a transfer is completet                            */
void DMA2_Channel3_IRQHandler(void) {

  if (DMA2->ISR & (1 << 9)) {             /* TCIF interrupt?                  */
    AD_last = (AD_last + 0x8000);         /* scale value to 0x0 .. 0xFFFF     */

    AD_done = 1;

    DMA2->IFCR  = (1 << 9);               /* clear TCIF interrupt             */
  }
}
#endif


#ifdef __SDADC_IRQ
/*  A/D IRQ: Executed when A/D Conversion is done                             */
void SDADC1_IRQHandler(void) {
  uint32_t tmp;

  if (SDADC1->ISR & (1 << 1)) {           /* SDADC1 injected EOC interrupt?   */
    tmp = SDADC1->JDATAR & 0xFFFF;
    AD_last = (uint16_t)(tmp + 0x8000);   /* scale value to 0x0 .. 0xFFFF     */

    AD_done = 1;
  }

}
#endif
